#!/bin/bash

###########################################################################################
#	run-distributed.sh
#		version: 	0.1	03/10/2017
#
#		author:		Gabriele D'Angelo	<g.dangelo@unibo.it>
#
#	description:
#		executes a single run of the wireless simulation on a set of networked hosts
#		(TCP/IP connectivity is required)
#		the hosts running the simulation are indicated in $HOSTS_FILE
#		the host running the SIMA is indicated in the first line of $HOSTS_FILE
#
#	usage:	
#		./run-distributed.sh <#SMH>
#
#		input parameters description:
#		#SMH				total number of wireless devices (nodes) to simulate
#						(e.g. each LP will simulate #SMH / #TOT_LP devices)
#
#		example: ./run-distributed.sh 1000
#			it executes a single run of a parallel simulation composed of 4 LPs in which
#			each LP models 250 nodes (i.e. 1000 / 4)
#			the allocation of LPs on hosts is defined in HOSTS_FILE, such as the 
#			SImulation MAnager (SIMA) host
#
#		output description:
#		- sima-$SIMA_HOST.log		SIMA log
#		- *-result.log				output of the first LP
#
###########################################################################################

###########################################################################################
# Simulation execution parameters
###########################################################################################
#
# 	Allocation of LPs on hosts
HOSTS_FILE="hosts-distributed.txt"
#
###########################################################################################

ESC="\033["

TOT_SMH=$1

###########################################################################################
# Cleaning up
function cleanup {
	for h in $HOST_LIST; do
		echo "...cleaning: $h"
		ssh $h "pkill -9 sima; pkill -9 wireless"
	done
}
###########################################################################################

if [ "$#" != "1" ]; then
        echo "		  Incorrect syntax...		 "
        echo "USAGE: $0 [#SMH]"
        echo ""
        exit
fi

echo "                          "
echo "                          "
echo -e "${ESC}29;39;1mWIRELESS SIMULATION ... ${ESC}0m"

echo "...parsing the LP-to-host (and SIMA) allocation file"

POS=0
TOT_LPS=-1
while IFS='' read -r line || [[ -n "$line" ]]; do
	HOSTNAME=`echo $line | cut -f1 -d':'`
	PRT=`echo $line | cut -f2 -d':'`
	LP=`echo $line | cut -f3 -d':'`
	PTH=`echo $line | cut -f4 -d':'`
	HOST[POS]=$HOSTNAME
	PORT[POS]=$PRT
	LPS[POS]=$LP
	EXE_PATH[POS]=$PTH
	TOT_LPS=$((TOT_LPS+LP))
	POS=$((POS+1))
done < "$HOSTS_FILE"

HOST_LIST=`echo "${HOST[@]}" | tr ' ' '\n' | sort -u | tr '\n' ' '`

# Intercept ctrl-c then cleaning
trap cleanup SIGINT

# Cleaning before starting
cleanup

# Partitioning the #SMH among the available LPs
SMH=$((TOT_SMH/TOT_LPS))

SIMA_HOST=${HOST[0]}
SIMA_PORT=${PORT[0]}
SIMA_PATH=${EXE_PATH[0]}

unset HOST[0]
unset PORT[0]
unset LPS[0]
unset EXE_PATH[0]

# Preparation of the wireless.ini configuration file
cp -f wireless-base.ini wireless.ini
SIMAHOST=`cat hosts-distributed.txt | head -1 | cut -f1 -d":"`
SIMAPORT=`cat hosts-distributed.txt | head -1 | cut -f2 -d":"`
echo "HOST=$SIMAHOST" >> wireless.ini
echo "PORT=$SIMAPORT" >> wireless.ini

echo "...running SIMA on $SIMA_HOST:$SIMA_PORT (waiting for $TOT_LPS LPs)"
# SImulation MAnager (SIMA) execution
ssh -n -f $SIMA_HOST "sh -c 'cd $SIMA_PATH; nohup ./sima $TOT_LPS > sima-$SIMA_HOST.log 2>&1 &'"

for i in `seq 1 ${#HOST[@]} `;
do
	for y in `seq 1 ${LPS[$i]}`;
	do
		echo starting "#LP:" $y/${LPS[$i]} on ${HOST[$i]}
		ssh ${HOST[$i]} "sh -c 'cd ${EXE_PATH[$i]}; nohup ./wireless $TOT_LPS $SMH 2> $i.err'" &
	done
done

echo "...waiting for LPs termination"
wait 

echo "...collecting statistics"
scp ${HOST[1]}:${EXE_PATH[1]}/0.out ${HOST[1]}-result.log

# Cleaning
rm -f wireless.ini

echo "...done"
